﻿local NekoLoot = LibStub("AceAddon-3.0"):NewAddon("NekoLoot", "AceEvent-3.0", "AceConsole-3.0")
NekoLoot:SetDefaultModuleLibraries("AceEvent-3.0", "AceConsole-3.0")

local AceConfReg = LibStub("AceConfigRegistry-3.0")
-- local LBF = LibStub("LibButtonFacade", true)
local L = LibStub("AceLocale-3.0"):GetLocale("NekoLoot")
-- local media = LibStub("LibSharedMedia-3.0")

local db, cachedLink, bindText, r, g, b, known


------
-- constants
local type, ipairs, unpack = type, ipairs, unpack

local defaults = {
	profile = {
		modules = { ["*"] = true },
		
		skin = {
			skinID = "NekoLoot",
			backdrop = true,
			gloss = 0,
			zoom = false,
			colors = {},
		},
		
		frame = {
			scale = 1.0,
			alpha = 1.0,
			strata = "DIALOG",
			
			lock = true,
			
			parent = UIParent,
			anchorPoint = "CENTER",
			parentAnchor = "CENTER",
			xOffSet = 0,
			yOffSet = 0,
			width = 200,
			
			backgroundColor = { 0, 0, 0, 0.7 },
			borderColor = { 0.5, 0.5, 0.5, 1 },
		},
		
		loot = {
			descToggle = true,
			descColor = { 1, 0.8, 0 },

			descFontSize = 10,
			nameFontSize = 12,
			bindFontSize = 12,

			expand = true,
			textWidth = 155,
			highlight = 1,
			
			height = 20,
			backgroundColor = { 0, 0, 0, 0.9 },
			buttonBorderColor = { 0.7, 0.7, 0.7, 1 },

			highlightButton = true,
			highlightFrame = false,

			-- no option yet
			frameBorderColor = { 0.5, 0.5, 0.5, 1 },
		},

		lootMode = {
			protectAgainstAuto = true,
			
			fast = false,
			extraFast = false,
			wasExtraFast = false,
			
			collapse = false,
			fastCollapse = true,
		},
		
		lootRoll = {},
	},
}


function NekoLoot:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("NekoLootDB", defaults, true)
	db = self.db.profile
	
	self:SetupOptions()
	
	-- create tooltip to scan for item binding
	self:CreateTooltip()

	self.buttonFrames = {}
	-- create loot frame, apply skin and add a single item button 
	self.lootFrame = NekoLoot:CreateLootFrame()
	self.buttonFrames[1] = NekoLoot:CreateLootToken(self.lootFrame, 1)
	self:ApplySettings()
	
	-- list of current lootable items
	self.items = {}
end


-- Add addon handler to events and disable blizzards defaults
function NekoLoot:OnEnable()
	-- disable default loot frame
	LootFrame:UnregisterEvent("LOOT_OPENED")
	LootFrame:UnregisterEvent("LOOT_SLOT_CLEARED")
	LootFrame:UnregisterEvent("LOOT_CLOSED")
	
	-- Register open of loot frame to event
	if db.lootMode.fast then
		self:ChangeLootOpen("fast", true)
	elseif db.lootMode.extraFast then
		self:ChangeLootOpen("extraFast", true)
	else
		self:ChangeLootOpen()
	end
	
	-- Register update of loot frame to event
	if db.lootMode.collapse then
		self:ChangeLootSlotCleared("collapse", true)
	elseif db.lootMode.fastCollapse then
		self:ChangeLootSlotCleared("fastCollapse", true)
	else
		self:ChangeLootSlotCleared()
	end
	
	-- Register to loot method changed. Used do display loot window when fast
	-- or extra fast looting is active
	self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED", "LootMethodChanged")
	self:LootMethodChanged()

	self:RegisterEvent("LOOT_CLOSED", "LootClosed")
	
	-- usefull events, may be support some times --
	-- Inventory is Full???
	
	-- UPDATE_BINDINGS
	-- AutoLootModified changed???
	
	-- CONFIRM_DISENCHANT_ROLL
	-- Fires when you try to roll "disenchant" for and item which Binds on Pickup. 
	
	-- CONFIRM_LOOT_ROLL
	-- Fires when you try to roll "need" or "greed" for and item which Binds on Pickup. 
	
	-- LOOT_BIND_CONFIRM
	-- Fired when the player attempts to take 'bind-on-pickup' loot
	
	-- "UPDATE_MASTER_LOOT_LIST"
	-- ???

	-- register for setting changed
	self.db.RegisterCallback(self, "OnProfileChanged", "ApplySettings")
	self.db.RegisterCallback(self, "OnProfileCopied", "ApplySettings")
	self.db.RegisterCallback(self, "OnProfileReset", "ApplySettings")

	-- TODO use shared media!
	-- media.RegisterCallback(self, "LibSharedMedia_Registered", "ApplySettings")
	-- media.RegisterCallback(self, "LibSharedMedia_SetGlobal", "ApplySettings")

	-- support for configuration mode
	CONFIGMODE_CALLBACKS = CONFIGMODE_CALLBACKS or {}
	CONFIGMODE_CALLBACKS["NekoLoot"] = function(action)
		if action == "ON" then
			self:Unlock(false)
		elseif action == "OFF" then
			self:Lock()
		end
	end
	
	self:ApplySettings()
end


-- Remove anything done by this addon and add blizzards defaults
function NekoLoot:OnDisable()
	self:UnregisterAllEvents()

	-- enable default loot frame
	LootFrame:RegisterEvent("LOOT_OPENED")
	LootFrame:RegisterEvent("LOOT_SLOT_CLEARED")
	LootFrame:RegisterEvent("LOOT_CLOSED")
end


-- Get Number of items of loot table and fill empty loot token in loot frame
-- if not enough loot token in loot frame add someone
function NekoLoot:OnUpdate()
end


-- Apply settings
function NekoLoot:ApplySettings()
	db = self.db.profile
	local options = db.frame

	self.lootFrame:SetFrameStrata(options.strata)
	self.lootFrame:SetScale(options.scale)
	self.lootFrame:SetAlpha(options.alpha)

	self.lootFrame:SetMovable(not options.lock)
	self.lootFrame:EnableMouse(not options.lock)
	
	-- Hintergrund setzen
	self:ApplySkin(NekoLootFrame, options.backgroundColor, options.borderColor)

	-- Ausrichtung
   	self.lootFrame:ClearAllPoints()
	self.lootFrame:SetPoint(
		options.anchorPoint,
		options.parent,
		options.parentAnchor,
		options.xOffSet,
		options.yOffSet
	)

	for _, lootToken in pairs(self.buttonFrames) do
		NekoLoot:ApplySetToLootToken(lootToken, db.loot)
	end
	
	if GetNumLootItems() > 0 then
		self:Update()
	else
		self:SetLootFrameHeight(1)
		self:SetLootFrameWidth(1, options.width)
	end
end


-- Apply setting to a single token frame
function NekoLoot:ApplySetToLootToken(lootToken, options)
	local button, font = lootToken.button, STANDARD_TEXT_FONT
	button.wrapper = self:ItemButtonWrapper(button, 6, 6)

	button.desc:SetVertexColor(unpack(options.descColor))
	button.desc:SetFont(font, options.descFontSize)
	button.text:SetFont(font, options.nameFontSize)
	button.bind:SetFont(font, options.bindFontSize, "OUTLINE")
	
	self:ApplySkin(lootToken, options.backgroundColor, options.buttonBorderColor)
end


function NekoLoot:GetModuleEnabled(module)
	return db.modules[module]
end


function NekoLoot:SetModuleEnabled(module, value)
	local old = db.modules[module]
	if old ~= value then
		db.modules[module] = value
		if value then
			self:EnableModule(module)
		else
			self:DisableModule(module)
		end
	end
end


function NekoLoot:ToggleLock(showUI)
	local func = not db.frame.lock and "Lock" or "Unlock"
	self[func](self, showUI)
	
	for _, module in NekoLoot:IterateModules() do
		module:ToggleLock(showUI)
	end
end


local function ConfigModeDialogHandler()
	if NekoLoot.dialog:IsShown() then
		NekoLoot.dialog:Hide()
		NekoLoot:ToggleLock()		
		AceConfReg:NotifyChange("NekoLoot")
	end
end


function NekoLoot:CreateConfigModeDialog()
	NekoLoot:CreateDialog(
		"NekoLootUnlockDialog",
		L["UnlockDialog/AllUnlocked"],
		L["UnlockDialog/Lock"],
		"BOTTOMRIGHT",
		ConfigModeDialogHandler)
end


-- Make window moveable
function NekoLoot:Unlock(showUI)
	db.frame.lock = false
	self.lootFrame:Show()
	self.lootFrame:EnableMouse(true)
	self.lootFrame:SetMovable(true)

	if showUI then self:CreateConfigModeDialog() end
end


-- Lock window
function NekoLoot:Lock()
	db.frame.lock = true
	
	if GetNumLootItems() < 1 then
		self.lootFrame:Hide()
	end
	
	self.lootFrame:EnableMouse(false)
	
	if self.dialog and (self.dialog:GetName() == "NekoLootUnlockDialog") then
		self.dialog:Hide()
	end
end


function NekoLoot:OnHide()
	-- if config mode keep frame visible
	if self.unlock_dialog and self.unlock_dialog:IsVisible() then return end

	NekoLootFrame:Hide()
end


function NekoLoot:OnShow()
	NekoLootFrame:Show()
end


-- set addiotonal information of an item
-- lootLink		: link of the item
-- bind			: text obj where binding should be set
-- desc			: text obj where description of the item should be set
function NekoLoot:SetSlotInfo(lootLink, bind, desc)
	local _, _, rarity, _, _, itemType, subType, _, equipLoc = GetItemInfo(lootLink)
	local bText, r, g, b = self:GetItemBinding(lootLink)
	
	-- set item binding text
	if bText ~= nil then
		bind:SetText(string.sub(bText, 1, 5))
		bind:SetVertexColor(r, g, b)
	end

	if itemType == ENCHSLOT_WEAPON then
		equipLoc = ENCHSLOT_WEAPON
	else
		equipLoc = _G[equipLoc]
	end

	if subType == "Junk" then 
		subType = (rarity > 0) and MISCELLANEOUS or subType
	end
	desc:SetText(((type(equipLoc) == "string" and equipLoc ~= "") and equipLoc..", " or "") .. ((subType == subType) and subType or subType.." "..itemType))
end


------
-- get item binding and the corresponding text color 
-- itemlink			: link of the item to scan
function NekoLoot:GetItemBinding(itemLink)
	if not itemLink then return nil end
	
	-- return cached information if possible
	if cachedLink == itemLink then
		return bindText, r, g, b, known
	else cachedLink = itemLink end
	
	local toolTip = self.tooltip
	local numLines, text, textLine

	toolTip:SetOwner(UIParent, "ANCHOR_NONE")
	toolTip:SetHyperlink(itemLink)
	numLines = NekoLootTT:NumLines()
	
	-- search only in the first six lines
	numLines = math.min( numLines, 6)
	
	-- search only for BoP, BoE, BoU and BtA
	bindText, known, r, g, b = "", false, 1, 1, 1
	for index = 1, numLines do
		textLine = _G["NekoLootTTTextLeft"..index]
		text = textLine:GetText()

		if text == ITEM_ACCOUNTBOUND or text == ITEM_BIND_TO_ACCOUNT then
			bindText, r, g, b = L["BtAcc"], .9, .8, .5
			break
		elseif text == ITEM_BIND_ON_EQUIP then
			bindText, r, g, b = L["BoE"], .3, 1, .3
			break
		elseif text == ITEM_BIND_ON_PICKUP then
			bindText, r, g, b = L["BoP"], 1, .0, .1
			break
		elseif text == ITEM_BIND_ON_USE then
			bindText, r, g, b = L["BoU"], .3, .5, 1
			break
		elseif text == ITEM_SPELL_KNOWN or text == USED then
			known = true
		end
	end
	toolTip:Hide()
	return bindText, r, g, b, known
end


-- Sets the height of the hole loot frame
-- numItems		: number of lootable items
function NekoLoot:SetLootFrameHeight(numItems)
	NekoLootFrame:SetHeight(20 + (numItems * (NekoLootButtonFrame1:GetHeight() + 2)))
end


-- sets width of the hole lootframe and of the severeal items to loot
-- numItems		: how many items
-- width		: width of the largest item
function NekoLoot:SetLootFrameWidth(numItems, width)
	local frame, frameBorder
	local frameWidth = db.frame.width
	local buttonWidth = -(db.frame.width - 37)

	-- replace width with dynamic frame width if expand is enabled
	if db.loot.expand then
		frameWidth = (self.buttonFrames[1].button:GetWidth() + width + 21)
		buttonWidth = -(width + 16)
	end

	-- resize anything
	for index = 1, numItems do
		frame = self.buttonFrames[index]
		frame:SetWidth(frameWidth)
		frame.button:SetHitRectInsets(0, buttonWidth, 0, -1)
		
		frameBorder = frame.border
		frameBorder:SetWidth(frameWidth * 1.72)
		frameBorder:ClearAllPoints()
		frameBorder:SetPoint("CENTER", frame, "CENTER", 5, 1)
	end
	self.lootFrame:SetWidth(frameWidth + 24)
end


-- Check if more items than buttons. Add missing buttons
-- numItems		: number of lootable items
function NekoLoot:CheckButtonNumber(numItems)
	while #self.buttonFrames < numItems do
		index = #self.buttonFrames + 1
		self.buttonFrames[index] = self:CreateLootToken(NekoLootFrame, index)
		self:ApplySetToLootToken(self.buttonFrames[index], db.loot)
	end
end


-- Set information of currency loot to the clickable button
-- frame		: frame of the button
-- button		: button where informations should be set
-- lootName		: lootName provided by wow api
local function SetCurrencyInformations(frame, button, lootName)
	-- set currency amount
	button.text:SetText(lootName:gsub('\n', ', '))
	button.text:SetPoint("LEFT", button, "CENTER", 25, 0)
	
	-- remove description and bind on
	button.desc:SetText("")
	button.bind:SetText("")
end


-- Set item information by scanning game tooltip
local function SetItemInformations(frame, button, lootName, hex)
	-- set formated item name
	button.text:SetText(('%s%s|r'):format(hex, lootName))
	
	-- additional item informations
	if db.loot.descToggle then
		-- set informations to description andn bind on textfield
		NekoLoot:SetSlotInfo(GetLootSlotLink(button.slot), button.bind, button.desc)
		-- reposition of item name and description
		button.text:SetPoint("LEFT", button, "CENTER", 25, db.loot.nameFontSize/2)
		button.desc:SetPoint("LEFT", button, "CENTER", 30, -(db.loot.descFontSize/2))
	else
		-- center item name
		button.text:SetPoint("LEFT", button, "CENTER", 25, 0)
	end
end


-- Set border colors of item, item button based on item quality color
-- frame		: frame of the button
-- button		: button where informations should be set
-- r, g, b		: color values
local function SetBorderColor(frame, button, rarity, _r, _g, _b)
	local r, g, b, a = unpack(db.loot.buttonBorderColor)
	-- set and show item icon color and/or item border color
	if rarity >= db.loot.highlight then
		if db.loot.highlightFrame then
			frame.border:SetVertexColor(_r, _g, _b)
			frame.border:Show()
		end
		-- border color based on rarity
		if db.loot.highlightButton then
			r, g, b, a = _r, _g, _b, 1
		end
	end
	-- text border
	frame:SetBackdropBorderColor(r, g, b, a)
	-- icon border
	button.wrapper:SetBackdropBorderColor(r, g, b, a)
end


function NekoLoot:Update()
	local buttonIndex, maxWidth = 0, 0
	local frame, button, text, desc, quality, bind
	local lootIcon, lootName, lootQuantity, rarity
	local r, g, b, hex, test, index
	
	local numItems = GetNumLootItems()
	local textWidth = (db.loot.expand and 700) or db.loot.textWidth
	
	self:CheckButtonNumber(numItems)
	
	for index = 1, numItems do
		lootIcon, lootName, lootQuantity, rarity, _ = GetLootSlotInfo(index);
		if lootIcon ~= nil then
			buttonIndex = buttonIndex + 1
			
			frame = self.buttonFrames[buttonIndex]
			self.items[buttonIndex] = frame
			frame:ClearAllPoints()
			if (index > 1) then
				frame:SetPoint("TOPLEFT", self.items[buttonIndex - 1], "BOTTOMLEFT", 0, -2)
			else
				frame:SetPoint("TOPLEFT", self.lootFrame, "TOPLEFT", 10, -10)
			end

			button = frame.button
			-- Don't break other addons so set default loot frame informations!
			button:SetID(index)
			button.slot = index
			button.quality = rarity
			
			r, g, b, hex = GetItemQualityColor(rarity)

			-- set icon
			SetItemButtonTexture(button, lootIcon)
			
			text = button.text
			text:ClearAllPoints()
			text:SetWidth(textWidth)
			text:SetVertexColor(r, g, b)

			desc = button.desc
			desc:ClearAllPoints()
			desc:SetWidth(textWidth)

			bind = button.bind

			SetItemButtonCount(button, lootQuantity)

			-- hide borders
			button.border:Hide()
			frame.border:Hide()
			
			if LootSlotIsCoin(index) or LootSlotIsCurrency(index) then
				SetCurrencyInformations(frame, button, lootName)
			else
				SetItemInformations(frame, button, lootName, hex)
			end

			maxWidth = max(maxWidth, text:GetStringWidth(), desc:GetStringWidth())
			
			SetBorderColor(frame, button, rarity, r, g, b)

			button:Show()
			frame:Show()
		end
	end
	
	-- hide unused button frames
	for index = buttonIndex + 1, #self.buttonFrames do
		self.buttonFrames[index]:Hide()
		self.buttonFrames[index].button:Hide()
	end
	
	-- set new height and width of loot window
	self:SetLootFrameHeight(buttonIndex)
	self:SetLootFrameWidth(buttonIndex, maxWidth)
	
	if numItems > 0 then self.lootFrame:Show() else self.lootFrame:Hide() end
end


-- EventHandler für einen einzelnes LootItem
function NekoLoot:OnEnter(self)
	if LootSlotIsItem(self.slot) then
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT", 32, 0)
		GameTooltip:SetLootItem(self.slot)
		if IsShiftKeyDown() then
			GameTooltip_ShowCompareItem()
		end
		CursorUpdate(self)
	end
end


function NekoLoot:StartMoving() 
	if not db.frame.lock then
		NekoLootFrame:StartMoving() 
	end
end


function NekoLoot:StopMoving()
	if not db.frame.lock then 
		NekoLootFrame:StopMovingOrSizing()
		local point, _, relativePoint, xOff, yOff = NekoLootFrame:GetPoint()

		self.db.profile.frame.anchorPoint = point
		self.db.profile.frame.parentAnchor = relativePoint
		self.db.profile.frame.xOffSet = xOff
		self.db.profile.frame.yOffSet = yOff
	end
	NekoLoot:ApplySettings()
end 
